#include "flip.h"

#define DRIVER_NAME "ripple"

DECLARE_PROPERTY(int,times,2);

PROPERTY_VECTOR_BEGIN
DECLARE_VECTOR(times,"flip in n times"),
PROPERTY_VECTOR_END
  ;


static int _timeout = 0;

/* these functions should be static */
static int timeout(){ return _timeout; }

static Uint32 getpixel32(SDL_Surface *surface, int x, int y)
{   
    int bpp = surface->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	return *(Uint32 *)p;
}

static void putpixel32(SDL_Surface *surface, int x, int y, Uint32 pixel)
{   
    int bpp = surface->format->BytesPerPixel;
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	*(Uint32 *)p = pixel;
}

static void ripple(SDL_Surface* screen, SDL_Surface* tmp, SDL_Surface* bmp,
				   timebase_context* t, int ratio1, int ratio2) {
	int size, half_size;
	int i, x, y;
	int bx, by;  /* border */
	int px, py;  /* previous */
	int half_w, half_h;
	Uint32 p;
	Uint8 r, g, b, r2, g2, b2;
	int ratio = ratio1 + ratio2;

	size = (screen->w > screen->h) ? screen->h : screen->w;
	half_size = size / 2;
	px = half_w = screen->w / 2;
	py = half_h = screen->h / 2;

	for (i = 1; i < half_size; i++) {
		bx = i * screen->w / size;
		by = i * screen->h / size;
		SDL_LockSurface(screen);
		for (x = half_w-bx; x <= half_w+bx; x++) {
			for (y = half_h-by; y <= half_h+by; y++) {
				if (x <= half_w+px && x >= half_w-px &&
					y <= half_h+py && y >= half_h-py) continue;

				p = getpixel32(tmp, x, y);
				SDL_GetRGB(p, tmp->format, &r, &g, &b);
				p = getpixel32(bmp, x, y);
				SDL_GetRGB(p, bmp->format, &r2, &g2, &b2);
				p = SDL_MapRGB(screen->format,
							   (r*ratio1+r2*ratio2)/ratio,
							   (g*ratio1+g2*ratio2)/ratio,
							   (b*ratio1+b2*ratio2)/ratio);
				putpixel32(screen, x, y, p);
			}
		}
		px = bx;
		py = by;

		SDL_UnlockSurface(screen);
		SDL_UpdateRect(screen, 0, 0, 0, 0);
		/*
		  褬ä٤ΤΩʤCPUեƯǡ
		  timebase_elapse(t, );
		*/
	}

}

#define TIMES_DEFAULT "2"

static void flip(SDL_Surface* screen, SDL_Surface* bmp)
{
	/* insert your majestic flip code here */
	timebase_context t;
	SDL_Surface* tmp;
	int i;
	timebase_init(&t);
	tmp = SDL_ConvertSurface(screen, screen->format, 0);

	for (i = 0; i < times; i++) {
		ripple(screen, tmp, bmp, &t, times-i-1, i+1);
	}

	return;
}

driver_vector ripple_vector = {
	DRIVER_NAME,
	"ripple?",
	flip,
	timeout, 
	property_init,
};
